# -*- coding: utf-8 -*-

import multiprocessing, subprocess
import os
from time import sleep
import logging
from functools import partial
#~ import psutil

from .products import L1cProduct, L2aProduct

logger = logging.getLogger("Multiprocessing")
logging.basicConfig(level=logging.INFO)
logger.setLevel(logging.INFO)

def multi(product):
    try:
        fwd = os.path.dirname(os.path.realpath(__file__))
        logger.info("Processing {}".format(product))
        cmd= ["setsid", "/usr/bin/python3", fwd + "/multiprocess_l2a.py", product]
        proc = subprocess.Popen(cmd)

        l1c = L1cProduct(product)
        l2a_identifier = l1c.identifier.replace("L1C_", "L2A_").replace("_OPER_", "_USER_")
        l2a_prod = L2aProduct(l2a_identifier)
        while not(l2a_prod.in_library):
            sleep(5)
        logger.info("End {}".format(product))
    except:
        logger.info("Plante {}".format(product))
        pass
    
def l2a_multiprocessing(process_list, nb_proc=4):
    """ """
    nb_proc = max(min(len(os.sched_getaffinity(0))-1, nb_proc), 1)
    pool = multiprocessing.Pool(nb_proc)
    results = [pool.map(multi, process_list)]
    pool.close()
    pool.join()
    return True

def multi_cldidx(indice_list, l2a_identifier):
    l2a = L2aProduct(l2a_identifier)
    l2a.process_cloud_mask_v2()
    l2a.process_indices(indice_list, True, True)

def cldidx_multiprocessing(process_list, indice_list=["NDVI", "NDWIGAO", "NDWIMCF"], nb_proc=4):
    """ """
    nb_proc = max(min(len(os.sched_getaffinity(0))-1, nb_proc), 1)
    pool = multiprocessing.Pool(nb_proc)
    results = [pool.map(partial(multi_cldidx, indice_list), process_list)]
    pool.close()
    pool.join()
    return True

def multi_cld(l2a_identifier):
    l2a = L2aProduct(l2a_identifier)
    try:
        l2a.process_cloud_mask_v2()
    except:
        pass

def cld_multiprocessing(process_list, nb_proc=4):
    """ """
    nb_proc = max(min(len(os.sched_getaffinity(0))-1, nb_proc), 1)
    pool = multiprocessing.Pool(nb_proc)
    results = [pool.map(multi_cld, process_list)]
    pool.close()
    pool.join()
    return True

def multi_idx(l2a_id_idx):
    l2a_identifier = l2a_id_idx[0]
    indice = [l2a_id_idx[1]]
    l2a = L2aProduct(l2a_identifier)
    try:
        l2a.process_indices(indice, True, True)
    except:
        pass

def idx_multiprocessing(process_list, nb_proc=4):
    """ """
    nb_proc = max(min(len(os.sched_getaffinity(0))-1, nb_proc), 1)
    pool = multiprocessing.Pool(nb_proc)
    results = [pool.map(multi_idx, process_list)]
    pool.close()
    pool.join()
    return True
